import hexToRGB from "helpers/hexToRGB";


export { };

const countColors = ({ pixels, width, height }) => {


  // This is the old method just by counting pixels
  const oldStore = {};

  //This method applies some weight to the pixels
  const store = {};
  let totalPixelValue = 0;

  let total = pixels.length / 4;

  const threshold = 200;

  for (let i = 0; i < total; i++) {


    const r = pixels[i * 4 + 0] > threshold ? 1 : 0;
    const g = pixels[i * 4 + 1] > threshold ? 1 : 0;
    const b = pixels[i * 4 + 2] > threshold ? 1 : 0;
    const id = r + "-" + g + "-" + b;

    //Caclulate y position of pixel
    const y = Math.floor(i / width);

    // KarlKilian Formel
    //const pixelValue = Math.cos(360 / (Math.pow(height, 2)) * Math.pow(y, 2) + (-360 / height) * y + 90);


    //const pixelValue = (2 * Math.sqrt(y * (height - y))) / height;

    // 0.2
    // 0.133
    const calibrationFaktor = 0.133;

    const pixelValue = Math.cos(
      (((360 / height ** 2) * y ** 2 + (-360 / height) * y + 90) / 360) *
      (2 + calibrationFaktor) *
      Math.PI
    );

    oldStore[id] = oldStore[id] + 1 || 1;

    if (id in store) {
      store[id] += pixelValue;
    } else {
      store[id] = pixelValue;
    }

    totalPixelValue += pixelValue;

  }

  const finalResult = {};


  // Normalize stores
  Object.keys(oldStore).forEach(k => {
    finalResult[k] = {
      distortedValue: oldStore[k] / total
    }
  });

  Object.keys(store).forEach(k => {
    finalResult[k].value = store[k] / totalPixelValue;
  });

  // Conver to array for easier 
  const result = Object.keys(finalResult).map(key => {
    const [r, g, b] = key.split("-").map((n) => parseInt(n) * 255);
    return {
      id: key,
      color: `rgb(${r},${g},${b})`,
      value: finalResult[key].value,
      distortedValue: finalResult[key].distortedValue
    }
  }).sort((a, b) => a.value > b.value ? -1 : 1)

  return result;

}

const distance2D = (x1, y1, x2, y2) => {
  const dx = x1 - x2;
  const dy = y1 - y2;
  return Math.sqrt(dx * dx + dy * dy);
}
const distance3D = (x1, y1, z1, x2, y2, z2) => {
  const dx = x1 - x2;
  const dy = y1 - y2;
  const dz = z1 - z2;
  return Math.sqrt(dx * dx + dy * dy + dz * dz);
}

const createFloodFillMap = ({ pixels, x, y, width, height }: { pixels: Uint8ClampedArray, x: number, y: number, width: number, height: number }) => {

  const pixelAmount = width * height;
  const index = (y * width + x) * 3;

  const _R = pixels[index + 0];
  const _G = pixels[index + 1];
  const _B = pixels[index + 2];

  let distanceWeight = 0.7;

  // This is the distance to the pixel that is furthest away from xy
  const maxDistance = distance2D((x < (width / 2)) ? width : 0, (y < (height / 2)) ? height : 0, x, y);

  // This the distance to the color that is furthest away from _R_G_B;
  let maxColorDistance = 0;

  for (let i = 0; i < pixelAmount; i++) {
    const r = pixels[i * 3 + 0];
    const g = pixels[i * 3 + 1];
    const b = pixels[i * 3 + 2];
    const dist = distance3D(r, g, b, _R, _G, _B);
    if (dist > maxColorDistance) maxColorDistance = dist;
  }


  const floodFillMap = new Uint8ClampedArray(pixelAmount);


  const maxWeight = (maxDistance * distanceWeight) + maxColorDistance * (1 - distanceWeight);

  for (let i = 0; i < pixelAmount; i++) {

    const _x = i % width;
    const _y = Math.floor(i / width);

    const r = pixels[i * 3 + 0];
    const g = pixels[i * 3 + 1];
    const b = pixels[i * 3 + 2];

    let dist = distance2D(_x, _y, x, y);
    let colDist = distance3D(r, g, b, _R, _G, _B);

    let weight = 1 - ((dist * distanceWeight) + (colDist * (1 - distanceWeight))) / maxWeight;

    floodFillMap[i] = Math.floor(weight * 255);

  }


  return floodFillMap;

}

const drawFloodFill = ({ floodMap, image, color, amount }: { floodMap: Uint8ClampedArray, image: Uint8ClampedArray, color: string, amount: number }) => {

  const [r, g, b] = hexToRGB(color);




  return image;

}

const send = (result, i) => {
  //@ts-ignore
  self.postMessage({ result, i });
}

self.addEventListener('message', function (e) {

  const data = e.data;

  const { i, type = "count" } = data;

  switch (type) {
    case "count":
      send(countColors(data), i);
      break;
    case "flood_map":
      send(createFloodFillMap(data), i);
      break;
    case "flood_fill":
      send(drawFloodFill(data), i);
      break;
    default:
      console.error("ERROR IN POXEL WORKER")
  }



}, false);